from fmatch.saas.operators.normalize_title import normalize_title


def test_title_rules():
    v = normalize_title("VP, Biz Dev (NA)")["value"]
    assert v["seniority"] in {"VP", "CXO", "Director", "Manager", "Owner", "IC"}
    assert isinstance(v["department"], str)

