# tests/test_overlay.py
import pytest


@pytest.mark.skip(reason="GUI refactored - _compute_tps function removed in async refactor")
def test_compute_tps():
    """
    This test was for the old synchronous GUI implementation.
    The async refactor removed the _compute_tps function as progress
    tracking is now handled differently through the async engine_bridge.
    """
    pass
