from dataclasses import dataclass
import os


@dataclass(frozen=True)
class TelemetryConfig:
    schema_version: str = "v2"
    client_sdk: str = os.getenv("CLIENT_SDK", "web")  # "web" | "gsheets" | "desktop"
    app_version: str = os.getenv("APP_VERSION", "0.1.0")
    consent_source: str = os.getenv("CONSENT_SOURCE", "settings")
    salt_env_var: str = "TELEMETRY_SALT"
