"""Optional Deal Desk approval readiness scout (v1 placeholder)."""

from __future__ import annotations

from typing import Any, Dict, List, Tuple

from .base import BaseScoutCheck, PreflightResult
from ..crm_provider import FilterExpr
from ..models import EvidenceRow


class DealDeskApprovalReadinessCheck(BaseScoutCheck):
    def preflight(self) -> PreflightResult:
        return PreflightResult(
            ok=False,
            unsupported=True,
            reason="Deal Desk approval readiness scout is not enabled in v1.",
        )

    def build_count_filters(self) -> Tuple[str, List[FilterExpr]]:
        return "Opportunity", []

    def build_sample_request(self) -> Tuple[str, List[str], List[FilterExpr], str | None, str, bool]:
        return "Opportunity", ["Id"], [], None, "ASC", False

    def evaluate(self, rows: List[Dict[str, Any]]) -> List[EvidenceRow]:
        return []


__all__ = ["DealDeskApprovalReadinessCheck"]
